#ifndef PLAYLISTMANAGER_H
#define PLAYLISTMANAGER_H

#define PAS 10
#define MAXFILE 10000

#include "QArray.h"

class CachedTag
{
private:
	CString sortPath;
	bool read;
	int duration;
	int genre;
	CString artist;
	CString album;
	CString comment;
	CString composer;
	CString playlist;
	CString title;
	CString year;
	TCHAR drive[MAX_PATH];
	TCHAR dir[MAX_PATH];
	TCHAR fname[MAX_PATH];
	TCHAR ext[MAX_PATH];
public:
	CachedTag();
	~CachedTag();
	CachedTag(CString nPlaylist);
	CachedTag(const CachedTag& copie);
	void copy(CachedTag copy);
	CString GetAlbum();
	CString GetArtist(bool fastDisplay = false);
	CString GetComment();
	CString GetComposer();
	CString GetDuration(bool fastDisplay = false);
	int GetDurationI();
	CString GetFilenameExt();
	CString GetGenre();
	int GetGenreI();
	CString GetPlaylist();
	bool GetRead();
	CString GetTag(int ShowingMode = 0, bool fastDisplay = false);
	CString GetTitle(bool fastDisplay = false);
	CString GetYear();
	CString GetDrive();
	CString GetDir();
	CString GetFilename();
	CString GetExt();
	CString GetSortPath();
	void ReadTag(bool forceRead = false);
	void SetAlbum(CString nAlbum);
	void SetArtist(CString nArtist);
	void SetComment(CString nComment);
	void SetComposer(CString nComposer);
	void SetDuration(int nDuration);
	void SetGenreI(int nGenre);
	void SetPlaylist(CString nPlaylist);
	void SetTitle(CString nTitle);
	void SetYear(CString nYear);
	void SetRead(bool nRead);
	void SetSortPath(CString nSortPath);
};

class PlaylistManager
{
private:
	CSortedArray<CachedTag, CachedTag&>* CachedTags;
	CSortedArray<CachedTag, CachedTag&>* sortedCachedTags;
	int ShowingMode;
public:
	CRITICAL_SECTION m_cs;
	int Current;
	PlaylistManager();
	~PlaylistManager();
	void OnStopAll();
	void Add(CString tag, int index = -1);
	void Add(CachedTag tag, int index = -1);
	CachedTag Get(int index);
	int GetIndex(CString filename);
	CString GetAlbum(int index);
	CString GetArtist(int index, bool fastDisplay = false);
	void SetArtist(int index, CString artist);
	CString GetComment(int index);
	void SetComment(int index, CString comment);
	CString GetComposer(int index);
	CString GetDuration(int index, bool fastDisplay = false);
	void SetDuration(int index, int nDuration);
	int GetDurationI(int index);
	CString GetFilenameExt(int index);
	CString GetGenre(int index);
	int GetGenreI(int index);
	CString GetPlaylist(int index);
	bool GetRead(int index);
	int GetShowingMode();
	int GetSize();
	CString GetTag(int index, bool fastDisplay = false);
	CString GetTitle(int index, bool fastDisplay = false);
	void SetTitle(int index, CString title);
	CString GetYear(int index);
	CString GetDrive(int index);
	CString GetDir(int index);
	CString GetFilename(int index);
	CString GetExt(int index);
	CString GetSortPath(int index);
	void InsertAt(CachedTag tag, int index);
	void PerformAutomation();
	void ReadAllTags();
	void ReadTagAT(int index, bool forceRead = false);
	void RemoveAll();
	void RemoveAt(int index);
	void SetShowingMode(int nShowingMode);
	bool StopPerformAutomation();
	bool StopReadAll();
	void QuickSort(BOOLEAN asc);
	int Compare(CachedTag& element1, CachedTag& element2);
	void RemoveDuplicateEntries();
	void Removene();
};
#endif

#define PLAYLIST_STOP_ALL			1024

extern PlaylistManager m_PlaylistManager;