///////////////////////////////////////////////////////////////////////////////
//
// MenuXP.h : header file
//
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <afxtempl.h>

#ifndef ODS_NOACCEL
#define ODS_NOACCEL 0x0100
#endif

///////////////////////////////////////////////////////////////////////////////
// Menu item image management class
class CImgDesc
{
public:
	HIMAGELIST m_hImgList;
	int m_nIndex;

	CImgDesc(HIMAGELIST hImgList = NULL, int nIndex = 0) : m_hImgList(hImgList), m_nIndex(nIndex)
	{
	}
};

///////////////////////////////////////////////////////////////////////////////
class CMenuXP
{
	// Operations
public:
	static void InitializeHook();
	static void UninitializeHook();

	static void SetXPLookNFeel(CWnd* pWnd, BOOL bXPLook = TRUE);
	static bool GetXPLookNFeel(const CWnd* pWnd);
	static void UpdateMenuBar(CWnd* pWnd);
	static void SetXPLookNFeel(CWnd* pWnd, HMENU hMenu, BOOL bXPLook = TRUE, bool bMenuBar = false, bool rMenuCaption = false);
	static bool IsOwnerDrawn(HMENU hMenu);
	static void SetMRUMenuBarItem(RECT& rc);
	static void SetMenuItemImage(UINT nID, HIMAGELIST hImgList, int nIndex);
	static void OnMeasureItem(MEASUREITEMSTRUCT* pMeasureItemStruct);
	static bool OnDrawItem(DRAWITEMSTRUCT* pDrawItemStruct, HWND hWnd);
	static LRESULT OnMenuChar(HMENU hMenu, UINT nChar, UINT nFlags);

	// Attributes
	static CMap <int, int, CString, CString&> ms_sCaptions;
	static CMap <int, int, CImgDesc, CImgDesc&> ms_Images;
protected:
	static CMap <HMENU, HMENU, CString, CString&> ms_sSubMenuCaptions;
	static CMap <HMENU, HMENU, CImgDesc, CImgDesc&> ms_SubMenuImages;

	friend class CMenuItem;
};

///////////////////////////////////////////////////////////////////////////////
// Menu item management class
//
class CMenuItem
{
protected:
	MENUITEMINFO m_miInfo;
	CString m_sCaption;
	CImgDesc m_ImgDesc;
	HIMAGELIST m_hImgList;
	int m_nIndex;

public:
	CMenuItem();
	CMenuItem(HMENU hMenu, UINT uItem, bool fByPosition = true);
	~CMenuItem();

	// Properties
public:
	int GetCaption(CString& sCaption) const;
	int GetShortCut(CString& sShortCut) const;
	bool GetSeparator() const;
	bool GetChecked() const;
	bool GetRadio() const;
	bool GetDisabled() const;
	bool GetDefault() const;
	HMENU GetPopup() const;
	UINT GetID() const;

	// Methods
public:
	int GetCaptionWidth(CDC* pDC) const;
	int GetShortCutWidth(CDC* pDC) const;
	int GetHeight(CDC* pDC) const;
	bool Draw(CDC* pDC, LPCRECT pRect, bool bSelected, bool bMenuBar = false, bool bHotLight = false, bool bInactive = false, bool bNoAccel = false) const;
	////////////////////////////////////////
	// 9 Jul 2003 begin mods Kris Wojtas
	////////////////////////////////////////
	void DrawGradient(CDC* pDC, COLORREF clrStart, COLORREF clrEnd, LPCRECT pRect, bool vertical = false) const;
	////////////////////////////////////////
	// 9 Jul 2003 end mods Kris Wojtas
	////////////////////////////////////////

public:
	static BYTE ms_nCheck;
	static CRect ms_rcMRUMenuBarItem;
};

///////////////////////////////////////////////////////////////////////////////
#define DECLARE_MENUXP()															 \
	protected:  																	 \
  afx_msg void OnInitMenuPopup(CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu);     \
  afx_msg void OnMeasureItem(int nIDCtl, LPMEASUREITEMSTRUCT lpMeasureItemStruct); \
  afx_msg void OnDrawItem(int nIDCtl, LPDRAWITEMSTRUCT lpDrawItemStruct);   	   \
  afx_msg LRESULT OnMenuChar(UINT nChar, UINT nFlags, CMenu* pMenu);

///////////////////////////////////////////////////////////////////////////////
#define ON_MENUXP_MESSAGES() \
  ON_WM_INITMENUPOPUP()    \
  ON_WM_MEASUREITEM()      \
  ON_WM_DRAWITEM()  	   \
  ON_WM_MENUCHAR()

///////////////////////////////////////////////////////////////////////////////
#define IMPLEMENT_MENUXP(theClass, baseClass)   								   \
	IMPLEMENT_MENUXP_(theClass, baseClass, CMenuXP::GetXPLookNFeel (this))

///////////////////////////////////////////////////////////////////////////////
#define IMPLEMENT_MENUXP_(theClass, baseClass, bFlag)   						   \
	void theClass::OnInitMenuPopup (CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu) \
	{   																		   \
	  baseClass::OnInitMenuPopup (pPopupMenu, nIndex, bSysMenu);				 \
		CMenuXP::SetXPLookNFeel (this, pPopupMenu->m_hMenu, 					   \
								 bFlag/* && !bSysMenu*/);   					   \
	}   																		   \
	void theClass::OnMeasureItem (int, LPMEASUREITEMSTRUCT lpMeasureItemStruct)    \
	{   																		   \
		CMenuXP::OnMeasureItem (lpMeasureItemStruct);   						   \
	}   																		   \
	void theClass::OnDrawItem (int nIDCtl, LPDRAWITEMSTRUCT lpDrawItemStruct)      \
	{   																		   \
  if ( !CMenuXP::GetXPLookNFeel(this) ) 					 \
		{   																	   \
			baseClass::OnDrawItem (nIDCtl, lpDrawItemStruct);   				   \
		}   																	   \
  if ( !CMenuXP::OnDrawItem (lpDrawItemStruct, m_hWnd) )			   \
		{   																	   \
			baseClass::OnDrawItem (nIDCtl, lpDrawItemStruct);   				   \
		}   																	   \
	}   																		   \
	LRESULT theClass::OnMenuChar (UINT nChar, UINT nFlags, CMenu* pMenu)		   \
	{   																		   \
		if ( CMenuXP::IsOwnerDrawn (pMenu->m_hMenu) )   						   \
		{   																	   \
			return CMenuXP::OnMenuChar (pMenu->m_hMenu, nChar, nFlags); 		   \
		}   																	   \
	  return baseClass::OnMenuChar (nChar, nFlags, pMenu);  					 \
	}

///////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.
